/* $Id: giccdlin.c,v 1.6 2011/03/03 09:34:26 rpalsa Exp $
 *
 * This file is part of the GIRAFFE Pipeline
 * Copyright (C) 2002-2008 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * $Author: rpalsa $
 * $Date: 2011/03/03 09:34:26 $
 * $Revision: 1.6 $
 * $Name: giraffe-2_8_8 $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <cxmessages.h>
#include <cxmemory.h>

#include <cpl_recipe.h>
#include <cpl_plugininfo.h>
#include <cpl_parameterlist.h>
#include <cpl_frameset.h>
#include <cpl_msg.h>

#include "giframe.h"
#include "gierror.h"
#include "giutils.h"

#include "irplib_detmon.h"
#include "irplib_detmon_lg.h"


static cxint giccdlin(cpl_parameterlist* config, cpl_frameset* set);


/*
 * Create the recipe instance, i.e. setup the parameter list for this
 * recipe and make it available to the application using the interface.
 */

static cxint
giccdlin_create(cpl_plugin* plugin)
{

    cpl_recipe* recipe = (cpl_recipe*)plugin;


    giraffe_error_init();


    /*
     * We have to provide the option we accept to the application. We
     * need to setup our parameter list and hook it into the recipe
     * interface.
     */

    recipe->parameters = cpl_parameterlist_new();
    cx_assert(recipe->parameters != NULL);


    /*
     * Fill the parameter list. All parameters are kept at the default values
     * with the expection of the tolerance and the pafname.
     */

    irplib_detmon_lg_fill_parlist(recipe->parameters, "giccdlin", "giraffe",
            "PTC",           /* --method */
            3,               /* --order */
            3,               /* --kappa */
            25,              /* --niter */
            -1,              /* --llx */
            -1,              /* --lly */
            -1,              /* --urx */
            -1,              /* --ury */
            10000,           /* --ref_level */
            "CPL_FALSE",     /* --intermediate */
            "CPL_FALSE",     /* --autocorr */
            "CPL_TRUE",      /* --collapse */
            "CPL_TRUE",      /* --rescale */
            "CPL_FALSE",     /* --pix2pix */
            "CPL_FALSE",     /* --bpmbin */
            -1,              /* --filter */
            26,              /* --m */
            26,              /* --n */
            0.1,             /* --tolerance */
            "CPL_FALSE",     /* --pafgen */
            "giccdlin",      /* --pafname */
            -1,              /* --llx1 */
            -1,              /* --lly1 */
            -1,              /* --urx1 */
            -1,              /* --ury1 */
            -1,              /* --llx2 */
            -1,              /* --lly2 */
            -1,              /* --urx2 */
            -1,              /* --ury2 */
            -1,              /* --llx3 */
            -1,              /* --lly3 */
            -1,              /* --urx3 */
            -1,              /* --ury3 */
            -1,              /* --llx4 */
            -1,              /* --lly4 */
            -1,              /* --urx4 */
            -1,              /* --ury4 */
            -1,              /* --llx5 */
            -1,              /* --lly5 */
            -1,              /* --urx5 */
            -1,              /* --ury5 */
            0,               /* --exts */
            OPT);


    if (cpl_error_get_code() != CPL_ERROR_NONE) {
        return 1;
    }

    return 0;

}


/*
 * Execute the plugin instance given by the interface.
 */

static cxint
giccdlin_exec(cpl_plugin* plugin)
{

    cpl_recipe* recipe = (cpl_recipe*)plugin;


    cx_assert(recipe->parameters != NULL);
    cx_assert(recipe->frames != NULL);

    return giccdlin(recipe->parameters, recipe->frames);

}


static cxint
giccdlin_destroy(cpl_plugin* plugin)
{

    cpl_recipe* recipe = (cpl_recipe*)plugin;


    /*
     * We just destroy what was created during the plugin initialization
     * phase, i.e. the parameter list. The frame set is managed by the
     * application which called us, so we must not touch it,
     */

    cpl_parameterlist_delete(recipe->parameters);

    giraffe_error_clear();

    return 0;

}


/*
 * The actual recipe starts here.
 */

static cxint
giccdlin(cpl_parameterlist* config, cpl_frameset* set)
{

    cpl_propertylist* ltable_properties = NULL;
    cpl_propertylist* gtable_properties = NULL;
    cpl_propertylist* ccube_properties  = NULL;
    cpl_propertylist* bpm_properties    = NULL;
    cpl_propertylist* acorr_properties  = NULL;
    cpl_propertylist* dflat_properties  = NULL;

    cpl_error_code status = CPL_ERROR_NONE;


    ltable_properties = irplib_detmon_fill_prolist("LINEAR_INFO",
                                                   NULL, NULL, CPL_FALSE);
    gtable_properties = irplib_detmon_fill_prolist("GAIN_INFO",
                                                   NULL, NULL, CPL_FALSE);
    ccube_properties = irplib_detmon_fill_prolist("COEFFS_CUBE",
                                                  NULL, NULL, CPL_FALSE);
    bpm_properties = irplib_detmon_fill_prolist("BAD_PIXEL_MAP",
                                                NULL, NULL, CPL_FALSE);
    acorr_properties = irplib_detmon_fill_prolist("AUTOCORR",
                                                  NULL, NULL, CPL_FALSE);
    dflat_properties = irplib_detmon_fill_prolist("DIFF_FLAT",
                                                  NULL, NULL, CPL_FALSE);

    status = irplib_detmon_lg(set,
                              config,
                              GIFRAME_LINEARITY_FLAT,
                              GIFRAME_LINEARITY_BIAS,
                              "giccdlin",
                              "giraffe",
                              REGEXP,
                              ltable_properties,
                              gtable_properties,
                              ccube_properties,
                              bpm_properties,
                              acorr_properties,
                              dflat_properties,
                              PACKAGE "/" PACKAGE_VERSION,
                              NULL, NULL, OPT);

    cpl_propertylist_delete(ltable_properties);
    cpl_propertylist_delete(gtable_properties);
    cpl_propertylist_delete(ccube_properties);
    cpl_propertylist_delete(bpm_properties);
    cpl_propertylist_delete(acorr_properties);
    cpl_propertylist_delete(dflat_properties);

    if (status != CPL_ERROR_NONE) {
        return 1;
    }

    return 0;

}


/*
 * Build table of contents, i.e. the list of available plugins, for
 * this module. This function is exported.
 */

int
cpl_plugin_get_info(cpl_pluginlist* list)
{

    cpl_recipe* recipe = cx_calloc(1, sizeof *recipe);
    cpl_plugin* plugin = &recipe->interface;


    cpl_plugin_init(plugin,
                    CPL_PLUGIN_API,
                    GIRAFFE_BINARY_VERSION,
                    CPL_PLUGIN_TYPE_RECIPE,
                    "giccdlin",
                    "Compute linearity and gain detector characteristics.",
                    "For detailed information please refer to the "
                    "GIRAFFE pipeline user manual.\nIt is available at "
                    "http://www.eso.org/pipelines.",
                    "Giraffe Pipeline",
                    PACKAGE_BUGREPORT,
                    giraffe_get_license(),
                    giccdlin_create,
                    giccdlin_exec,
                    giccdlin_destroy);

    cpl_pluginlist_append(list, plugin);

    return 0;

}
